@@ -7,22 +7,19 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <sys/stat.h>
 
 #include <stdio.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
 #include <unistd.h>
 
-#ifdef __STDC__
-#include <stdlib.h>
-#else
-extern double atof();
-extern char* getenv();
-#endif
-
 #include "paper.h"
 
 struct paper {
@@ -30,29 +27,6 @@ struct paper {
     double pswidth, psheight;
 };
 
-#if __STDC__ - 0 == 0
-#define const
-#define void
-#endif
-
-#ifndef PAPERSIZEVAR
-#define PAPERSIZEVAR	"PAPERSIZE"
-#endif
-
-#ifndef PAPERSIZE
-#define PAPERSIZE	"/etc/papersize"
-#endif
-
-#ifndef PAPERVAR
-#define PAPERVAR	"PAPERCONF"
-#endif
-
-/* The following choice should be in the papers[] array. */
-
-#ifndef PAPERCONF
-#define PAPERCONF	"letter"
-#endif
-
 /* This table will be searched in order: please put typical paper sizes
    at the beginning of it. */
 
@@ -72,32 +46,17 @@ int paperdone(void) {
     return 0;
 }
 
-#if __STDC__ - 0 == 0
-char* papername(spaper)
-    struct paper* spaper;
-#else
 const char* papername(const struct paper* spaper)
-#endif
 {
     return spaper->name;
 }
 
-#if __STDC__ - 0 == 0
-double paperpswidth(spaper)
-    struct paper* spaper;
-#else
 double paperpswidth(const struct paper* spaper)
-#endif
 {
     return spaper->pswidth;
 }
 
-#if __STDC__ - 0 == 0
-double paperpsheight(spaper)
-    struct paper* spaper;
-#else
 double paperpsheight(const struct paper* spaper)
-#endif
 {
     return spaper->psheight;
 }
@@ -117,42 +76,44 @@ const struct paper* paperlast(void) {
     return lastpaper;
 }
 
-#if __STDC__ - 0 == 0
-struct paper* papernext(spaper)
-    struct paper* spaper;
-#else
 const struct paper* papernext(const struct paper* spaper)
-#endif
 {
     return (++spaper)->name ? spaper : 0;
 }
 
-#if __STDC__ - 0 == 0
-struct paper* paperprev(spaper)
-    struct paper* spaper;
-#else
 const struct paper* paperprev(const struct paper* spaper)
-#endif
 {
     return spaper == papers ? 0 : --spaper;
 }
 
 const char* defaultpapersizefile(void) {
-    return PAPERSIZE;
-}
-
-const char* systempapersizefile(void) {
-    const char* papersize = getenv(PAPERSIZEVAR);
-    return papersize ? papersize : defaultpapersizefile();
-}
-
-const char* defaultpapername(void) {
     return PAPERCONF;
 }
 
+const char* systempapersizefile(void) {
+    const char* paperconf = getenv(PAPERCONFVAR);
+/* 
+Previously PAPERCONFVAR used to contain a paper name and PAPERSIZEVAR
+contained a file path.  Now they're reversed.  If we don't find a '/'
+in PAPERCONFVAR, fall-back to the old behaviour.
+*/
+
+    if ((paperconf != NULL) && (strchr(paperconf, '/') == NULL)) {
+	paperconf = getenv(PAPERSIZEVAR);
+	if ((paperconf != NULL) && (strchr(paperconf, '/') == NULL))
+	    paperconf = NULL;
+    }
+
+    return paperconf ? paperconf : defaultpapersizefile();
+}
+
+const char* defaultpapername(void) {
+    return PAPERSIZE;
+}
+
 char* systempapername(void) {
+    const char* paperconf;
     char* paperstr;
-    char* papersize;
     char* paperenv;
     const char* paperdef;
     FILE* ps;
@@ -160,7 +121,20 @@ char* systempapername(void) {
     const struct paper* pp;
     int c;
 
-    if ((paperenv = getenv(PAPERVAR))) {
+/* 
+Previously PAPERSIZEVAR used to contain a file path and PAPERCONFVAR
+contained a paper name.  Now they're reversed.  If we find a '/' in
+PAPERSIZEVAR, fall-back to the old behaviour.
+*/
+
+    paperenv = getenv(PAPERSIZEVAR);
+    if ((paperenv != NULL) && (strchr(paperenv, '/') != NULL)) {
+	paperenv = getenv(PAPERCONFVAR);
+	if ((paperenv != NULL) && (strchr(paperenv, '/') != NULL))
+	    paperenv = NULL;
+    }
+
+    if (paperenv) {
         paperstr = malloc((strlen(paperenv) + 1) * sizeof(char));	
 	
 	if (! paperstr) return 0;
@@ -171,14 +145,13 @@ char* systempapername(void) {
 	    return strcpy(paperstr, paperenv);
     }
 
-    papersize = getenv(PAPERSIZEVAR);
+    paperconf = systempapersizefile();
+    if (paperconf && stat(paperconf, &statbuf) == -1) return 0;
+    
+    if (!paperconf) paperconf = defaultpapersizefile();
 
-    if (papersize && stat(papersize, &statbuf) == -1) return 0;
-
-    if (!papersize) papersize = PAPERSIZE;
-
-    if ((stat(papersize, &statbuf) != -1) &&
-	(ps = fopen(papersize, "r"))) {
+    if ((stat(paperconf, &statbuf) != -1) &&
+	(ps = fopen(paperconf, "r"))) {
 
         while ((c = getc(ps)) != EOF) {
 	    if (c == '#') {
@@ -214,7 +187,10 @@ char* systempapername(void) {
 		fclose(ps);
 
 		paperstr = malloc((strlen(papername) + 1) * sizeof(char));
-		if (! paperstr) return 0;
+		if (! paperstr) {
+		    free(papername);
+		    return 0;
+		}
 
 		strcpy(paperstr, papername);
 		free(papername);
@@ -236,12 +212,7 @@ char* systempapername(void) {
         return 0;
 }
 
-#if __STDC__ - 0 == 0
-struct paper* papeinfo(paper)
-    char* paper;
-#else
 const struct paper* paperinfo(const char* paper)
-#endif
 {
     const struct paper* pp;
 
@@ -254,13 +225,7 @@ const struct paper* paperinfo(const char* paper)
     return 0;
 }
 
-#if __STDC__ - 0 == 0
-struct paper* paperwithsize(pswidth, psheight)
-    double pswidth;
-    double psheight;
-#else
 const struct paper* paperwithsize(double pswidth, double psheight)
-#endif
 {
     const struct paper* pp;
 
